/*
 * GDevelop Core
 * Copyright 2008-2016 Florian Rival (Florian.Rival@gmail.com). All rights
 * reserved. This project is released under the MIT License.
 */

#include "StandardEvent.h"
#include "GDCore/CommonTools.h"
#include "GDCore/Events/CodeGeneration/EventsCodeGenerationContext.h"
#include "GDCore/Events/CodeGeneration/EventsCodeGenerator.h"
#include "GDCore/Events/Serialization.h"
#include "GDCore/IDE/Dialogs/EventsEditor/EventsEditorItemsAreas.h"
#include "GDCore/IDE/Dialogs/EventsEditor/EventsEditorSelection.h"
#include "GDCore/IDE/Dialogs/EventsEditor/EventsRenderingHelper.h"
#include "GDCore/Serialization/SerializerElement.h"
#include "GDCore/TinyXml/tinyxml.h"

using namespace std;

namespace gd {

StandardEvent::StandardEvent() : BaseEvent() {}

StandardEvent::~StandardEvent(){};

vector<const gd::InstructionsList*> StandardEvent::GetAllConditionsVectors()
    const {
  vector<const gd::InstructionsList*> allConditions;
  allConditions.push_back(&conditions);

  return allConditions;
}

vector<const gd::InstructionsList*> StandardEvent::GetAllActionsVectors()
    const {
  vector<const gd::InstructionsList*> allActions;
  allActions.push_back(&actions);

  return allActions;
}
vector<gd::InstructionsList*> StandardEvent::GetAllConditionsVectors() {
  vector<gd::InstructionsList*> allConditions;
  allConditions.push_back(&conditions);

  return allConditions;
}

vector<gd::InstructionsList*> StandardEvent::GetAllActionsVectors() {
  vector<gd::InstructionsList*> allActions;
  allActions.push_back(&actions);

  return allActions;
}

void StandardEvent::SerializeTo(SerializerElement& element) const {
  gd::EventsListSerialization::SerializeInstructionsTo(
      conditions, element.AddChild("conditions"));
  gd::EventsListSerialization::SerializeInstructionsTo(
      actions, element.AddChild("actions"));
  gd::EventsListSerialization::SerializeEventsTo(events,
                                                 element.AddChild("events"));
}

void StandardEvent::UnserializeFrom(gd::Project& project,
                                    const SerializerElement& element) {
  gd::EventsListSerialization::UnserializeInstructionsFrom(
      project, conditions, element.GetChild("conditions", 0, "Conditions"));
  gd::EventsListSerialization::UnserializeInstructionsFrom(
      project, actions, element.GetChild("actions", 0, "Actions"));
  gd::EventsListSerialization::UnserializeEventsFrom(
      project, events, element.GetChild("events", 0, "Events"));
}

/**
 * Render the event in the bitmap
 */
void StandardEvent::Render(wxDC& dc,
                           int x,
                           int y,
                           unsigned int width,
                           gd::EventsEditorItemsAreas& areas,
                           gd::EventsEditorSelection& selection,
                           const gd::Platform& platform) {
#if !defined(GD_NO_WX_GUI)
  gd::EventsRenderingHelper* renderingHelper = gd::EventsRenderingHelper::Get();
  int border = renderingHelper->instructionsListBorder;

  // Draw event rectangle
  wxRect rect(x,
              y,
              renderingHelper->GetConditionsColumnWidth() + border,
              GetRenderedHeight(width, platform));
  renderingHelper->DrawNiceRectangle(dc, rect);

  renderingHelper->DrawConditionsList(
      conditions,
      dc,
      x + border,
      y + border,
      renderingHelper->GetConditionsColumnWidth() - border,
      this,
      areas,
      selection,
      platform);
  renderingHelper->DrawActionsList(
      actions,
      dc,
      x + renderingHelper->GetConditionsColumnWidth() + border,
      y + border,
      width - renderingHelper->GetConditionsColumnWidth() - border * 2,
      this,
      areas,
      selection,
      platform);

  // Make sure that Render is rendering an event with the same height as
  // GetRenderedHeight : Use same values for border and similar calls to compute
  // heights
#endif
}

unsigned int StandardEvent::GetRenderedHeight(
    unsigned int width, const gd::Platform& platform) const {
#if !defined(GD_NO_WX_GUI)
  if (eventHeightNeedUpdate) {
    gd::EventsRenderingHelper* renderingHelper =
        gd::EventsRenderingHelper::Get();
    int border = renderingHelper->instructionsListBorder;

    // Get maximum height needed
    int conditionsHeight = renderingHelper->GetRenderedConditionsListHeight(
        conditions,
        renderingHelper->GetConditionsColumnWidth() - border * 2,
        platform);
    int actionsHeight = renderingHelper->GetRenderedActionsListHeight(
        actions,
        width - renderingHelper->GetConditionsColumnWidth() - border * 2,
        platform);

    renderedHeight =
        (conditionsHeight > actionsHeight ? conditionsHeight : actionsHeight) +
        border * 2;
    eventHeightNeedUpdate = false;
  }

  return renderedHeight;
#else
  return 0;
#endif
}

}  // namespace gd
